/*-------------------------------------------------------------------
   code03e8.c - code resource #1000, the first hack (SysAppLaunch)
-------------------------------------------------------------------*/

#include <PalmOS.h>

#include "KioskHack.h"
#include "resource.h"

/* Copyright (c) 2005 Raymond Ingles.

   This is free software; you can redistribute it and/or modify it under the
   terms of the GNU General Public License as published by the Free Software
   Foundation; either version 2, or (at your option) any later version.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
   or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.  */

/*##################################################################
  Intercept SysAppLaunch, and if we're locked, redirect it to relaunch
  the current 'locked' app. We only do this when the new app would get
  new globals; that way, a 'locked' app can still call others as a
  subroutine if needed.
  ##################################################################*/
Boolean PW_Unset_P(UInt32 *hash);

asm ( " bra MySysAppLaunch" );

Err MySysAppLaunch (UInt16 cardNo, LocalID dbID, UInt16 launchFlags, 
		     UInt16 cmd, MemPtr cmdPBP, UInt32* resultP)
{
  Err (*oldtrap)(UInt16, LocalID, UInt16, UInt16, MemPtr, UInt32*);

  if (launchFlags & sysAppLaunchFlagNewGlobals) {
    Kiosk_Prefs prefs;
    UInt16 prefs_size = sizeof(prefs);

    /* Attempting to run a new app. Are we locked? */
    if (noPreferenceFound ==
        PrefGetAppPreferences(HACK_ID, 0, &prefs, &prefs_size, true)) {
        /* Yikes! Not configured at all! */
        SndCommandType snd_cmd;
        snd_cmd.cmd = sndCmdFreqDurationAmp;
        snd_cmd.param3 = PrefGetPreference(prefAlarmSoundVolume);
        snd_cmd.param2 = TONE_DURATION;
        snd_cmd.param1 = TONE_MED;
        SndDoCmd(0, &snd_cmd, false);
        snd_cmd.param1 = TONE_LOW;
        SndDoCmd(0, &snd_cmd, false);
    } else {
      /* Okay, prefs exist. */
      if (prefs.locked_p) {
        if (!dbID || PW_Unset_P((UInt32 *)prefs.passwd_hash)) {
          /* Yikes! Not configured right! */
          SndCommandType snd_cmd;
          snd_cmd.cmd = sndCmdFreqDurationAmp;
          snd_cmd.param3 = PrefGetPreference(prefAlarmSoundVolume);
          snd_cmd.param2 = TONE_DURATION;

          if (!dbID) {
            /* dbID should *never* be zero. */
            snd_cmd.param1 = TONE_LOW;
            SndDoCmd(0, &snd_cmd, false);
          }
          if (PW_Unset_P((UInt32 *)prefs.passwd_hash)) {
            snd_cmd.param1 = TONE_MED;
            SndDoCmd(0, &snd_cmd, false);
          }
        } else {
          /* Okay, the prefs exist and appear to be valid. Use 'em. */
          switch (cmd) {
          case sysAppLaunchCmdSystemLock:
            if (!prefs.blocksecr_p) {
              /* User doesn't want us to interfere with this. */
              goto calltrap;
            }
            break;
          case sysAppLaunchCmdPanelCalledFromApp:
          case sysAppLaunchCmdReturnFromPanel:
            if (!prefs.blockpanl_p) {
              /* User doesn't want us to interfere with this. */
              goto calltrap;
            }
            break;
          default:
            break;
          }
          /* If we're here, that's a no-no! */
          if (prefs.beeplaunch_p) {
            SndCommandType snd_cmd;
            snd_cmd.cmd = sndCmdFreqDurationAmp;
            snd_cmd.param3 = PrefGetPreference(prefAlarmSoundVolume);
            snd_cmd.param2 = TONE_DURATION;
            snd_cmd.param1 = TONE_HIGH;
            SndDoCmd(0, &snd_cmd, true);
          }
          cardNo = prefs.AppCardNo;
          dbID = prefs.AppDbID;
          cmd = sysAppLaunchCmdNormalLaunch;
          cmdPBP = NULL; /* memory will be freed by OS, I hope. */
        }
      } else {
        /* Record this as the new 'current app'. */
        prefs.AppCardNo = cardNo;
        prefs.AppDbID = dbID;
        PrefSetAppPreferences(HACK_ID, 0, 1, &prefs, sizeof(prefs), true);
      }
    }
  } else if (sysAppLaunchCmdSystemReset == cmd) {
    /* The system was reset. Unlock the Palm, or bad things could happen. */
    Kiosk_Prefs prefs;
    UInt16 prefs_size = sizeof(prefs);

    if (noPreferenceFound !=
        PrefGetAppPreferences(HACK_ID, 0, &prefs, &prefs_size, true) &&
        prefs.locked_p) {
      prefs.locked_p = 0;
      PrefSetAppPreferences(HACK_ID, 0, 1, &prefs, sizeof(prefs), true);
    }
  }
 calltrap:
  /* Call the actual systrap. */
  FtrGet(HACK_ID, HACKINDEX1, (UInt32*)&oldtrap );
  return oldtrap(cardNo, dbID, launchFlags, cmd, cmdPBP, resultP);
}

Boolean PW_Unset_P(UInt32 *hash)
{
  int i;

  for (i=0; i<4; i++) {
    if (hash[i]) {
      return false;
    }
  }
  return true;
}
